/** 
 * Objekt Avatar
 * 
 * verwaltet die Eigenschaften eines Avatars
 * 
 * @author lray
 * @version 2007-02-20
 */       

  var Avatar = Class.create();
  Avatar.prototype = {
  	oNewAvatar:Object(),
    oNewAvatarC1:Object(),
    oNewAvatarC2:Object(),
    oNewAvatarDetails:Object(),    
    oNewAvatarDetailsNode:Array(),
    oNewAvatarDetailsPic: Object(),
    bTmpActive: Boolean(false),

    oUser:Object(),
    
    oLogger:Object(),   
  	iSecondsForShowingChat: Number(90),
  	iMovingType:Number(0),
  	
    initialize: function(oItem) {
      this.oLogger = new Log4js.getLogger("vtntTest");
      this.oLogger.debug("Avatar initiated");
   
      this.oNewAvatar = document.createElementNS(svgNS,'g'); 
      // Create the loudness-circle of the avatar    
  	  this.oNewAvatarC1 = document.createElementNS(svgNS,'circle');		
      this.oNewAvatarC1.setAttribute('cx',0);
      this.oNewAvatarC1.setAttribute('cy',0);
      this.oNewAvatarC1.setAttribute('class','ava_loudness');
      this.oNewAvatar.appendChild(this.oNewAvatarC1);
  	  
      // Create the body-circle of the avatar    
  	  this.oNewAvatarC2 = document.createElementNS(svgNS,'circle');		
      this.oNewAvatarC2.setAttribute('cx',0);
      this.oNewAvatarC2.setAttribute('cy',0);
      this.oNewAvatarC2.setAttribute('r',5);
      this.oNewAvatarC2.setAttribute('class','ava_body');       
      this.oNewAvatar.appendChild(this.oNewAvatarC2);    
          
      if (oItem instanceof User) {
        this.setUser(oItem);
        }
      else {
        this.oLogger.info("Avatar is fresh");
        this.oUser = new User();
        }

    },

    /**
    * setUser
    *
    * falls im lokalen Objekt die "LastMoved"-Property auf -1 steht, wurde
    * sie seit dem letzten Aufruf lokal geändert, und bleibt daher aktuell.
    *
    * @param User Nutzerobjekt
    * @author lray
    * @version 2007-03-02
    */
    setUser: function(oItem) {
    	this.oUser = oItem;
        this.oLogger.debug("Avatar is getting "+this.oUser.getName()+" and xpos "+this.oUser.getX());
	    
	    this.updateComponent();
    },

    updateComponent: function() {
      this.setId(this.getId());
      this.setPosition(this.getX(), this.getY());

      this.setName(this.getName());
      this.setCompany(this.getCompany());
      this.setPositionName(this.getPositionName());
      this.setPicUrl(this.getPicUrl());
      this.setOwn(this.isOwn());
      this.setActive(this.isActive());
      this.setLoudness(this.getLoudness());
      this.setEmail(this.getEmail());
      this.setLastSeen(this.getLastSeen());
      this.setLastMoved(this.getLastMoved());
      this.setLastAction(this.getLastAction());
      this.setBlocked(this.getBlocked());

      this.fillUserDetails();
    },

    getComponent: function() {
      return this.oUser;
    },

    notify: function() {
      this.oLogger.warn("Class "+this.getId()+" notified, but not yet implemented.");
    },

    remove: function() {
      this.oNewAvatar.parentNode.removeChild(this.oNewAvatar);
    },

  	setMovingType: function(item) { 
      this.iMovingType = item;
      switch (this.iMovingType) {     
        case "0":
              this.oNewAvatar.setAttribute('pointer-events','all', false);
              this.oNewAvatarC1.setAttribute('pointer-events','visibleStroke', false);
              this.oNewAvatarC2.setAttribute('pointer-events','visiblePainted', false);
          break;
        case "1":
              this.oNewAvatar.setAttribute('pointer-events','none', false);
              this.oNewAvatarC1.setAttribute('pointer-events','none', false);
              this.oNewAvatarC2.setAttribute('pointer-events','none', false);
          break;
        case "2":
              this.oNewAvatar.setAttribute('pointer-events','none', false);
              this.oNewAvatarC1.setAttribute('pointer-events','none', false);
              this.oNewAvatarC2.setAttribute('pointer-events','none', false);
          break;
        default:
              this.oNewAvatar.setAttribute('pointer-events','all', false);
              this.oNewAvatarC1.setAttribute('pointer-events','visibleStroke', false);
              this.oNewAvatarC2.setAttribute('pointer-events','visiblePainted', false);
          break;
      } 
    },

  	getMovingType: function() { 
          return this.iMovingType;
    },

	setPosition: function(xpos, ypos) {
        try {
	    	if (!this.oNewAvatar.getAttribute('transform')) 
				throw("translate not yet set");
			
			// loesche alle aktuellen Bewegungen
			this.clearScope('avaMove'+this.getId());
        
        new Effect.SVGMove(this.oNewAvatar, 
          { 
			x: xpos,
			y: ypos, 
			mode: 'absolute', 
			duration: 2.0,  
			transition: Effect.Transitions.slowstop,
			queue: { 
				position:'end',
				scope: 'avaMove'+this.getId(),
				limit:1
				}
			}
		);
        this.oLogger.debug("Set Fading-Effect onto Radius of User "+this.getId());
      } catch (e) { 
		this.setX(xpos);
		this.setY(ypos);    
		this.updatePosition();
	  }
	},
    
   /**
    * updatePosition
    *
    * setzt die aktuelle Position entweder auf die bergebenen Parameter oder
    * die Position des dekorierten User-Objektes
    *
    * @param Number x - X-Achse
    * @param Number y - Y-Achse
    * @author lray
    */
    updatePosition: function(x,y) {
      x = Number((x || this.oUser.getX()));
      y = Number((y || this.oUser.getY()));
      this.oNewAvatar.setAttribute('transform','translate('+String(x+', '+y)+')');    
    },

    /**
    * clearScope
    *
    * entfernt alle Effect-Queues mit der uebergebenen ID
    *
    * @param String name - Name der zu leerenden Queue
    * @author lray
    */
    clearScope: function(name) {
		var queue = Effect.Queues.get(name);
		queue.each(function(e) { e.cancel() });
    },
    
    stopAnimation: function() {
    	this.clearScope('avaMove'+this.getId());
    	this.updatePosition();
    },
    
    setX: function(xpos) { 
      this.oUser.setX(xpos);
	  if (!this.oNewAvatar.getAttribute('transform')||this.isOwn())
      	this.updatePosition();
    },
    
  	getX: function() { 
      return this.oUser.getX();
    },

  	setY: function(ypos) {
    	this.oUser.setY(ypos);
		if (!this.oNewAvatar.getAttribute('transform')|| this.isOwn())
      		this.updatePosition();
    },
    
  	getY: function() {return this.oUser.getY();},

  	setName: function(name) {
      this.oUser.setName(name);
      this.oNewAvatarC2.setAttribute('title', this.oUser.getName()+" ("+this.oUser.getId()+")");
    },
  	getName: function() {return this.oUser.getName();},

  	setBlocked: function(blocked) {
      this.oUser.setBlocked(blocked);
    },
  	getBlocked: function() {return this.oUser.getBlocked();},

  	setNick: function(name) {
      this.oUser.setNick(name);
    },
  	getNick: function() {return this.oUser.getNick();},

  	setCompany: function(company) {
      this.oUser.setCompany(company);
    },
  	getCompany: function() {return this.oUser.getCompany();},

  	setPositionName: function(posName) {
      this.oUser.setPositionName(posName);
    },
  	getPositionName: function() {return this.oUser.getPositionName();},

  	setPicUrl: function(picUrl) {
      this.oUser.setPicUrl(picUrl);
    },
    
  	getPicUrl: function() {
		return this.oUser.getPicUrl();	
  	},

  	setId: function(id) {
      this.oUser.setId(id);
      this.oNewAvatar.setAttribute('id',this.oUser.getId());
      this.oNewAvatarC1.setAttribute( 'id', this.oUser.getId()+'ava_loudness');
      this.oNewAvatarC2.setAttribute( 'id', this.oUser.getId()+'Body');
      
      this.oNewAvatarC2.setAttribute('title', this.oUser.getName()+" ("+this.oUser.getId()+")");  
    },
  	getId: function() {return this.oUser.getId();},
  
  	setLoudness: function(loudness) {
      this.oUser.setLoudness(loudness);
      this.oNewAvatarC1.setAttribute('r',this.oUser.getLoudness());  
    },
  	
    getLoudness: function() {return this.oUser.getLoudness();},

  	setActive: function(active) {
		if (Boolean(active) != this.bTmpActive) {
		this.clearScope('avaFade'+this.getId());
      	if (this.oUser.isActive()) {
         try { 
            if (this.isOwn())
          		new Effect.Appear(this.oNewAvatarC1, {
          		    duration: 1.0,
                  from: 0.0, 
                  to: 1.0,
          			queue: { 
          				position:'end',
          				scope: 'avaFade'+this.getId(),
          				limit:1
          				}
       
              	}
              );
            else
              new Effect.Appear(this.oNewAvatarC1, {
		            duration: 1.0,
                    from: 0.0,
                    to: 0.2, 
              			queue: { 
              				position:'end',
              				scope: 'avaFade'+this.getId(),
              				limit:1
              			}
               });
           } catch (e) {};
        } else {  
          try { 
             new Effect.Fade( this.oNewAvatarC1, {
            		 duration: 1.0,
            		 from: 1.0,
            		 to: 0.0,
             		 queue: { 
            		  position:'end',
            			scope: 'avaFade'+this.getId(),
            			limit:1
            		}
      	     });
          } catch (e) {};
          this.oNewAvatarC1.setAttribute('visibility','hidden');
        }
      }
      this.oUser.setActive(Boolean(active));
	  if (this.oUser.isActive())
     		this.oNewAvatarC1.setAttribute('visibility','visible');
      else
           this.oNewAvatarC1.setAttribute('visibility','hidden');	  
	  this.bTmpActive= Boolean(active);
    },
  	
    isActive: function() {return this.oUser.isActive();},

  	setOwn: function(own) {
      this.oUser.setOwn(own);
      if (this.oUser.isOwn()) {
        this.oNewAvatar.setAttribute('class','avatar moveable');
        this.oNewAvatarC1.setAttribute('style','opacity: 1.0;');
        
        
      } else {
        this.oNewAvatar.setAttribute('class','avatar');
      }
    },
  	
    isOwn: function() {
      return this.oUser.isOwn();
    },

    myCallBackOnStart: function() {
       //alert("starting Effect.");
    },
    
    // Was heisst "iSec" ?
    setLastAction: function(lastAction) {
      var Id = this.getId();
      this.oLogger.debug("AvatarAsDecorator.setLastAction: neue lastAction fuer "+this.getId()+":"+lastAction);
      //lastAction-max=80 sec, also differenz errechnen
      //if (!this.isOwn())
     	this.clearScope('avaFade'+this.getId());
        try {
          var newFrom = (lastAction/100)+0.2;
          if (newFrom >1.0 ) newFrom = 1.0;
          
          var newDuration = 30.0*newFrom;
          
          new Effect.Fade(this.oNewAvatarC1, { 
        		duration: newDuration,
        		from: newFrom,
        		to: (this.isOwn()?0.5:0.2),
        		queue: { 
        			position:'end',
        			scope: 'avaFade'+this.getId(),
        			limit:2
        		}
      	  });
          this.oLogger.debug("Set Fading-Effect onto Radius of User "+this.getId());
        } catch (e) {
        // to be filled later
        }
      this.oUser.setLastAction(lastAction);
    },
  	getLastAction: function() {return this.oUser.getLastAction();},

  	setLastMoved: function(lastMoved) {this.oUser.setLastMoved(lastMoved)},
  	getLastMoved: function() {return this.oUser.getLastMoved();},

  	setEmail: function(item) {this.oUser.setEmail(item);},
  	getEmail: function() {return this.oUser.getEmail();},

    /**
     *setLastSeen
     *haelt den Timestamp des letzten Signals vom Nutzer direkt
     *
     * @param Date lastSeen - Timestamp
     * @author: lray
     * @version:  2007-02-14
     **/                             
  	setLastSeen: function(lastSeen) {this.oUser.setLastSeen(lastSeen);},
  	getLastSeen: function() {return this.oUser.getLastSeen();},
  	
  	showUserData: function() {
      this.oLogger.info("showUserData"+this.oNewAvatarDetails);
      if (this.oNewAvatarDetails != '[object SVGGElement]') {
        this.oNewAvatarDetails = document.createElementNS(svgNS,'g');
        this.oNewAvatarDetails.setAttribute('id',this.oUser.getId()+'details');
        this.oNewAvatarDetails.setAttribute('class','ava_details');
        this.oNewAvatarDetails.setAttribute('transform','translate(5,5)');        
        
        var oNewAvatarDetailsBg = document.createElementNS(svgNS,'rect');
        oNewAvatarDetailsBg.setAttribute('width','170');
        oNewAvatarDetailsBg.setAttribute('height','80');
        this.oNewAvatarDetails.appendChild(oNewAvatarDetailsBg);        

        this.oNewAvatarDetailsPic = document.createElementNS(svgNS,'image');
        this.oNewAvatarDetailsPic.setAttribute('x',5);
        this.oNewAvatarDetailsPic.setAttribute('y',5);    
        this.oNewAvatarDetailsPic.setAttribute('width',64);
        this.oNewAvatarDetailsPic.setAttribute('height',72);
        this.oNewAvatarDetailsPic.setAttributeNS(xlinkNS,'href',this.oUser.getPicUrl());
        this.oNewAvatarDetails.appendChild(this.oNewAvatarDetailsPic);
        
        this.fillUserDetails();

        this.oNewAvatar.appendChild(this.oNewAvatarDetails);
      }
        this.oNewAvatarDetails.setAttribute('visibility','visible');
    },

  	hideUserData: function() {
      this.oLogger.info("hideUserData");
      this.oNewAvatarDetails.setAttribute('visibility','hidden');
    },
       
    fillUserDetails: function() {
      if (this.oNewAvatarDetails == '[object SVGGElement]') {      
        var detailData = new Array( 
                                this.oUser.getName(),
                                this.oUser.getNick(),
                                this.oUser.getCompany(),
                                this.oUser.getPositionName(),
                                this.oUser.getPicUrl()
                         );
        this.oNewAvatarDetailsNode = new Array();
               
        for (var i=0; i<4;i++){
          this.oNewAvatarDetailsNode[i] = document.createElementNS(svgNS,'text');
          var oNewAvatarDetailsTextNode = document.createTextNode(detailData[i]);
          this.oNewAvatarDetailsNode[i].setAttribute('x',70);
          this.oNewAvatarDetailsNode[i].setAttribute('y',15+(i*15));    
          this.oNewAvatarDetailsNode[i].appendChild(oNewAvatarDetailsTextNode);
          this.oNewAvatarDetails.appendChild(this.oNewAvatarDetailsNode[i]);
        };
        this.oNewAvatarDetails.removeChild(this.oNewAvatarDetailsPic);
        this.oNewAvatarDetailsPic.setAttributeNS(xlinkNS,'href',this.oUser.getPicUrl());
        this.oNewAvatarDetails.appendChild(this.oNewAvatarDetailsPic);
        //this.oNewAvatarDetailsPic.setAttributeNS(xlinkNS,'href',this.oUser.getPicUrl());
      } else this.oLogger.info("AvatarAsDecorator.fillUserDetails: noch kein showUserData durchgefhrt.");
    },
    
    paint: function() {  
      return this.oNewAvatar;
    }
}
