/**
 * Der View stellt die Daten des Models am Bildschirm dar. Er ermglicht
 * also die Sicht auf das Model und reagiert auf nderungen im Model. 
 *
 * Stellt entsprechende Avatare in SVG dar.
 * 
 * @author lray
 * @version 2007-02-20  
 */  

//  copyPrototype(myTest, myTest2);
// my second OO-JavaScript
  var AvatarView = Class.create();

  AvatarView.prototype = {
    elements: Array(), // Liste von Nutzern 
    oAvatarCollection: Object(), //Sammlung der entsprechenden Avatare
    oLogger:Object(),  
    sName: String("Instance of AvatarCollection"),
    
    oParent: Object(),   
    oModel: Object(),
      
    initialize: function(oModel, sName) {
      this.oLogger = new Log4js.getLogger("vtntTest");
     
      this.oAvatarCollection = document.createElementNS(svgNS,'g');
      this.oAvatarCollection.setAttribute('id','chatUser');
      
      if (sName != null) this.sName = sName;
      
      if (oModel == null) {   
        this.oLogger.warn("AvatarCollection: kein Modell angegeben,"
                         +" vTnT laeuft im Leerlauf!");
      } else {

        this.oLogger.debug(this.toString()+" initiated");
        this.oModel = oModel;
        this.oModel.addObserver(this);      
      }
    },
    
    add: function (item) {
      if (item instanceof Avatar) {
        this.oAvatarCollection.appendChild(item.paint());
        this.elements.push(item);
        this.oLogger.debug("AvatarCollection.add: added Avatar "+item.getName());        
      } else {
          if (item instanceof User) {
            oItemDecorator = new Avatar(item);
            this.oAvatarCollection.appendChild(oItemDecorator.paint());
            this.elements.push(oItemDecorator);      
            this.oLogger.debug("AvatarCollection.add: added User "
                                +item.getName());            
            this.oLogger.debug("AvatarCollection.add: added User "+item.getName());
          } else this.oLogger.warn("AvatarCollection.add: unbekannter Objekt-Typ");
     }
    }, 

    _each: function(iterator) {
        for ( var i = 0; i < this.elements.length; i++)
          iterator(this.elements[i]);
    },   
    
    paint: function() {
      return this.oAvatarCollection;
    },
    
    toString: function() {
      return this.sName;
    },
    
    setParent: function(item) {
      this.oParent = item;
    },

    /**
     * Ueberschreiben der Interfacemethode update des
     * Observer, legt die Reaktion auf Aenderungen fest.
     * @param m Modell, welches Aenderungen meldet
     * @param o Art der nderung (derzeit Array von Strings)
     */
    notify: function( m, o)
    {
     //alert("avatar-notification:"+o["user"].length+"/"+this.elements.length);
      if( this.oModel == m ) { 
        if ((o["user"] instanceof Array) && (o["user"].length > 0)) {
            this.updateUsers(o["user"]);
            this.ownAvatarOnTop();
        }
        //else this.oLogger.debug("AvatarCollection.notify: no new data (Object "+typeof(o)+") ist Array? "+ (o instanceof Array));      
        this.updateView();
      }     
    },
    
    updateUsers: function(aUserID) {
//       try {
        var newElements = aUserID;
        
        for (var i =0; i < newElements.length; i++) {
          var bChanged = false;
          var tmpElements;
          for (var j=0; j < this.elements.length; ++j ) {
            if (this.elements[j].getId() == newElements[i]) {
              bChanged = true;
              this.elements[j].updateComponent()
              this.oLogger.debug("AvatarCollection.updateUsers: "
                   +Number(i+1)+"/"+newElements.length+": "
                   +this.elements[j].getName()+"(angepasst)");
              break;
            }
          } 
          if (!bChanged) {
            tmpElements = new Avatar(this.oModel.getUserById(newElements[i]));
            this.elements.push(tmpElements);
            this.oAvatarCollection.appendChild(tmpElements.paint());
            this.oLogger.debug("AvatarCollection.updateUsers: "
               +Number(i+1)+"/"+newElements.length+": "
               +tmpElements.getName()+"(eingefuegt)");
          }
        }
    },
    
    /**
     * Entfernt Nutzer, welche die Penalty berschritten hat
     * 
     * @author lray
     * @version 2007-02-27
     */
     updateView: function() {
        for (var i=(this.elements.length-1); i>=0 ; i--) {
            if (!this.oModel.getUserById(this.elements[i].getId())) {
              this.oAvatarCollection.removeChild(this.elements[i].paint())
              this.elements.splice(i,1);
            } else {
              //this.oAvatarCollection.appendChild(this.elements[i].paint());
            }
        }
     },                              


   /**
    * ownAvatarOnTop
    *     
    * SVG doesn't support z-index like in css, that's why all own Avatars have
    * to be put on top by hand.
    * 
    * @author lray
    * @version 2007-02-23
    */                       
   ownAvatarOnTop: function() {
      for (var i=0; i< this.elements.length; i++) {
        if (this.elements[i].isOwn()) {
          this.oAvatarCollection.removeChild(this.elements[i].paint());
          this.oAvatarCollection.appendChild(this.elements[i].paint());         
          } 
      }
   }, 
    
  }
  Object.extend(AvatarView.prototype, Enumerable);

    function copyPrototype(descendant, parent) {
        var sConstructor = parent.toString();
        var aMatch = sConstructor.match( /\s*function (.*)\(/ );
        if ( aMatch != null ) { descendant.prototype[aMatch[1]] = parent; }
        for (var m in parent.prototype) {
            descendant.prototype[m] = parent.prototype[m];
        }
    };

