/**
 * class SliderController
 *   
 * Der Controller ist die Eingabeschnittstelle zwischen Benutzer und Modell.
 * Er interpretiert die empfangenen Eingabedaten und bergibt sie dem Modell.
 * Ein Controller ist meist auf ein spezielles View zugeschnitten. Ist sein 
 * Verhalten vom Model abhngig, so muss er auch auf nderungen im Model
 * reagieren knnen.
 * 
 * Dieser Control ermoeglicht die Steuerun gdes Sliders zur Zoom-Level-
 * Einstellung der Avatar-Map.  
 * 
 * @author lray
 * @version 2007-03-28
 */    

  var SliderControl = Class.create();

  SliderControl.prototype = {

    oModel: Object(),
    oLogger:Object(),  
    sName: String("SliderController"),
    oUser: Object(),
    svgdoc:Object(),
    start:Boolean(false),
    startx:Number(-1),
    startx2:Number(31),
    endex:Number(100),
    slr: Number(),
    txr: Number(),
    thx: Number(),

  /**
   * Initialisiert Model und View.
   * @param oMap zum Controller gehoeriger View
   * @param m Mathematisches Modell
   */
    initialize: function( oModel, sName )
    {
      this.oLogger = new Log4js.getLogger("vtntTest");

      if (oModel instanceof vTnTModel) {
        this.oModel = oModel;
      } else this.oLogger.warn("SliderControl.initialize: Modell nicht bergeben, laufe im Leerlauf!")
      
      if (sName != null) this.sName = sName;
    },

  /**
   * Setzt Model und View zurck.
   */
    release: function()
    {
      this.oModel = null;
    },

    toString: function()
    {
      return this.sName;
    },
    
    getSVGDoc:function (load_evt)
    {
      this.svgdoc=load_evt.target.ownerDocument;
    },

    init: function() {
      try {
	      this.slr=this.svgdoc.getElementById("slider").getElementsByTagName("rect")[0];
	      this.txr=this.svgdoc.getElementById("slider").getElementsByTagName("text")[0];    
	  } catch (e) {
	      this.oLogger.warn("sliderControl.init"+e);
	  }
    },

    moveSlider:function (mousemove_event)
    {  
      var r=mousemove_event.clientX-this.startx;
      if(r>100) r=100;

      if(this.start && r>=0 && r<=this.endex)
      {
        this.slr.setAttribute("x",this.startx2+r);
       // this.oModel.setZoomLevel(0.2+(r*0.03));
        this.txr.firstChild.data=r;
      }
    },

    releaseSlider:function (mousemove_event)
    {  
      var r = mousemove_event.clientX-this.startx;
      if(r>100)r=100;

      if(r>=0 && r<=this.endex)
      {
        this.slr.setAttribute("x",this.startx2+r);
        this.oModel.setZoomLevel(0.2+(r*0.03));
        this.oLogger.info("sliderControl.releaseSlider: setze zoomLevel auf"+(0.2+(r*0.03)));
        this.txr.firstChild.data=r;
        this.startx = - 1;
      }
    },

  /* --------------------------------------------------*/
  // handleEvent-Methode
  
  /** This is high-level function.
   * It must react to delta being more/less than zero.
   */
  handle:function (delta) {
          if (delta > 0)
  		this.oModel.setZoomLevel(this.oModel.getZoomLevel()*1.1);
          else
  		this.oModel.setZoomLevel(this.oModel.getZoomLevel()*0.9);
  },

  /** Event handler for mouse wheel event.
   */
    wheel:function (event){
          var delta = 0;
          if (!event) /* For IE. */
                  event = window.event;
          if (event.wheelDelta) { /* IE/Opera. */
                  delta = event.wheelDelta/120;
                  /** In Opera 9, delta differs in sign as compared to IE.
                   */
                  if (window.opera)
                          delta = -delta;
          } else if (event.detail) { /** Mozilla case. */
                  /** In Mozilla, sign of delta is different than in IE.
                   * Also, delta is multiple of 3.
                   */
                  delta = -event.detail/3;
          }
          /** If delta is nonzero, handle it.
           * Basically, delta is now positive if wheel was scrolled up,
           * and negative, if wheel was scrolled down.
           */
          if (delta)
                  sliderView.getControl().handle(delta);
          /** Prevent default actions caused by mouse wheel.
           * That might be ugly, but we handle scrolls somehow
           * anyway, so don't bother here..
           */
          if (event.preventDefault)
                  event.preventDefault();
  	event.returnValue = false;
  },

  setStartX:function(startX) {
	this.startx = startX;
  },

}
