/** 
 * class VisualMap
 * 
 * @author lray
 * @version 2007-02-20
 */

  var VisualMap = Class.create();

  VisualMap.prototype = {
   
    elements: Array(), //Collections von Avataren o.. (also die Sammlung der Sammlung von Avataren)
    oItemCollection: Object(),
    oLogger:Object(),
    iWidth: Number(0),
    iHeight: Number(0),
    iScale: Number(1),
    iTranslationX: Number(0),
    iTranslationY: Number(0),
    sName: String("unbenannt"),
    iViewMode: Number(1),
    
    /**
     * MVC - Modell/Control
     */         
    oModel: Object(),
    oControl: Object(),
   
    /**
     * function initialize
     * initialisiert den View.
     * 
     * @param Object oModel - zugrundeliegendes Modell
     * @author lray
     * @version 2007-02-16
     */                                          
    initialize:function(oModel,sName) {
      this.oLogger = new Log4js.getLogger("vtntTest");

      this.oItemCollection = document.createElementNS(svgNS,'svg');
      this.oItemCollection.setAttribute('width',this.getWidth());
      this.oItemCollection.setAttribute('height',this.getHeight());
      this.oItemCollection.setAttribute('id','vTnTMap');

      if (sName != null) this.sName = sName;

      if (oModel == null) {
        
        this.oLogger.warn("visualMap: kein Modell angegeben, vTnT laeuft im Leerlauf!");
      } else {

        this.oLogger.debug("VisualMap initiated");
        this.oModel = oModel;  
        this.oControl = this.makeControl( oModel, this);
        //this.oControl.initialize( oModel, this); 
      };
      this.oModel.addObserver(this);
      this.updateViewBox();
    },
    
    /**
     * Ueberschreiben der Interfacemethode update des
     * Observer, legt die Reaktion auf Aenderungen fest.
     * @param m Modell, welches Aenderungen meldet
     * @param o Art der nderung (derzeit Array von Strings)
     */
    notify: function( m, o)
    {
      if( this.oModel == m ) { 
        if (!isNaN(parseInt(o.viewMode)) ) {
            this.oLogger.debug("VisualMap.notify: viewMode "+o.viewMode+"("+typeof(o.viewMode)+")");
            this.setViewMode(o.viewMode);
        }
        
        if (!isNaN(parseFloat(o.zoomLevel)) ) {
            this.oLogger.debug("VisualMap.notify: zoomLevel "+o.zoomLevel+"("+typeof(o.zoomLevel)+")");
            
            this.setCurrentScale(o.zoomLevel);
            this.modifyCSS();
        }      

      }     
    },


   /**
   * Erzeugt ein Control-Componente.   
   * @return Control fuer View
   */
    makeControl: function( oModel, oMap ) {
        return new AvatarControl( oModel, oMap );
    },

    getControl: function() {
      //alert(this.oControl.toString());
      return this.oControl;
    },

   /**
   * Erzeugt und startet View.
   */
    makeView: function() {
      this.paint(); 
      this.oControl.start();
    },
    
    /*
    * Gibt die Views der Reihe nach zurck.
    *
    * TODO: Mega-Buggy, da fest implementiert!
    * Es gibt mittlerweile mehrere Views innerhalb der VisualMap:
    *   Avatare
    *   Hintergrund-Karte
    *   Slider rechts unten
    *   
    * @author lray
    * @version 2007-03-28           
    */    
    getView: function() {
      return this.elements[1];
    },
    
    /**
   * Setzt Model und Controller zurck.
   */
    release:function() {
      this.oControl.release();
      this.oControl = null;
      this.oModel.deleteObserver( this);
      this.oModel = null;
    },
     
    add: function (item) {
      this.oItemCollection.appendChild(item.paint());
      item.setParent(this);
      this.elements.push(item);
      this.oLogger.debug("VisualMap.add: View "+item.toString()+" and hold now "+this.elements.length+" Objects");
      this.oLogger.debug("VisualMap.added "+item.paint());
      this.updateViewBox();
    }, 

    _each: function(iterator) {
        for ( var i = 0; i < this.elements.length; i++)
          iterator(this.elements[i]);
    },   
    
    setAttribute: function (key, value) {
      this.oItemCollection.setAttribute(key, value);
    },
    
    getAttribute: function(key) {
      return this.oItemCollection.getAttribute(key);
    },
    
    /**
     * Sets the ViewMode for this Chat
     * 
     * the ViewMode might be something like "minimized", or "maximum".
     * While in Viewmode "minimized" (0), the window is more squeezed.
     * also, the own User should always be in the middle, right?
     * 
     * @author lray
     * @version 2007-03-29                                       
     */         
    setViewMode:function(iViewMode) {
     
	  var vTnTWidth = 175-8; //210;
      var vTnTHeight = 220-8; //210;
      var vTnTMaxWidth = 745;  //420
      var vTnTMaxHeight = 360; //360
      var vTnTVisualMaxWidth = 420;
      var vTnTVisualMaxHeight = 360;      
      
      this.iViewMode = iViewMode;  
      switch (iViewMode) {
        case 0: 
              this.setWidth(vTnTWidth);
              this.setHeight(vTnTHeight);
              
              /*this.setTranslationX(this.oModel.getCenter()[0]-(90/this.oModel.getZoomLevel()));                       
              this.setTranslationY(this.oModel.getCenter()[1]-(90/this.oModel.getZoomLevel()));*/
              this.setToCenter();
              
              if (this.oLogger.isInfoEnabled())
                this.oLogger.info("visualMap.setViewMode(0): Zoom="+this.oModel.getZoomLevel()
                                  +" center ("+this.oModel.getCenter()[0]+","+this.oModel.getCenter()[1]+")"
                                  +" new viewbox: "+this.getTranslationX()+" "+this.getTranslationY()+" 180, 350");
              break;
        case 1: 
              this.setWidth(vTnTVisualMaxWidth);
              this.setHeight(vTnTVisualMaxHeight); 
              
              this.setToCenter();
              /*this.setTranslationX(this.oModel.getCenter()[0]-(Math.round(this.getWidth/2)/this.oModel.getZoomLevel()));                       
              this.setTranslationY(this.oModel.getCenter()[1]-(Math.round(this.getHeight()/2)/this.oModel.getZoomLevel()));*/
              if (this.oLogger.isInfoEnabled())
                this.oLogger.info("visualMap.setViewMode(1): Zoom="+this.oModel.getZoomLevel()
                                  +" center ("+this.oModel.getCenter()[0]+","+this.oModel.getCenter()[1]+")"
                                  +" new viewbox: "+this.getTranslationX()+","+this.getTranslationY()+" 420, 360");

              break;
        default: 
      }
      this.updateViewBox();
    },
    
    setToCenter: function() {
       this.setTranslationX(this.oModel.getCenter()[0]-(Math.round(this.getWidth()/2)/this.oModel.getZoomLevel()));                       
       this.setTranslationY(this.oModel.getCenter()[1]-(Math.round(this.getHeight()/2)/this.oModel.getZoomLevel()));      
      return true;
    },


    getViewMode: function() {
      return this.iViewMode;
    },

    setWidth: function(width) {
      this.iWidth = width;
      this.oItemCollection.setAttribute('width',this.getWidth());
      this.updateViewBox();
    },

    getWidth: function() {
      return this.iWidth;
    },

    setHeight: function(height) {
      this.iHeight = height;
      this.oItemCollection.setAttribute('height',this.getHeight());
      this.updateViewBox();
    },

    getHeight: function() {
      return this.iHeight;
    },

    setTranslationX: function(transX) {
      this.iTranslationX = transX;
      this.oItemCollection.currentTranslate.x = transX;   
      this.oModel.setTranslationX(transX);
      this.updateViewBox();
    },

    getTranslationX: function() {
      return this.iTranslationX;
    },

    setTranslationY: function(transY) {
      this.iTranslationY = transY;
      this.oItemCollection.currentTranslate.y = transY;
      this.oModel.setTranslationY(transY);
      this.updateViewBox();
    },

    getTranslationY: function() {
      return this.iTranslationY;
    },

    /**
     *setCurrentScale
     *
     * skaliert die Anzeige auf das entsprechende Format, so da der eigene 
     * Avatar an der entsprechenden Stelle stehen bleibt.
     * 
     * @author lray
     * @version 2007-03-28                         
     */         
    setCurrentScale: function(zoom) {
      var tmpScale = this.iScale;
      var tmpOwnPosition = this.oModel.getCenter();
      var tmpOwnX = tmpOwnPosition[0];
      var tmpOwnY = tmpOwnPosition[1];
      var tmpTransX = this.getTranslationX();
      var tmpTransY = this.getTranslationY();
      
      var tmpNewTransX = (tmpOwnX - tmpTransX )* (1+(tmpScale - zoom));
      var tmpNewTransY = (tmpOwnY - tmpTransY )* (1+(tmpScale - zoom));            
      
      this.setTranslationX(Math.round(tmpOwnX - tmpNewTransX));
      this.setTranslationY(Math.round(tmpOwnY - tmpNewTransY));      
      this.oLogger.info("Scale:"+tmpScale+"->"+zoom
                        +" transX:"+tmpTransX+"->"+Math.round(tmpOwnX - tmpNewTransX)
                        +" transY:"+tmpTransY+"->"+Math.round(tmpOwnY - tmpNewTransY)
                        );

      this.iScale = zoom;    
      this.oItemCollection.currentScale = zoom;
      this.updateViewBox();      
    },
    
    getCurrentScale: function() {
      return this.iScale;
    },

    getSVGPoint: function() {
      return this.oItemCollection.createSVGPoint();
    },
    
    updateViewBox: function() {
      this.oItemCollection.setAttribute('viewBox',
             this.getTranslationX()+' '
            +this.getTranslationY()+' '
            +Math.round((this.getWidth()/this.getCurrentScale()))+' '
            +Math.round((this.getHeight()/this.getCurrentScale())));
 	 
 	  this.oModel.setWidth(this.getTranslationX() + Math.round(this.getWidth()/this.getCurrentScale()));
 	  this.oModel.setHeight(this.getTranslationX() + Math.round(this.getHeight()/this.getCurrentScale()));
 	  this.oModel.setTranslationX(this.getTranslationX());
 	  this.oModel.setTranslationY(this.getTranslationY());
    }, 
  
    paint: function() {    
      this.modifyCSS();
      return this.oItemCollection;
    },
  
    modifyCSS: function() {
        // test for IE
        var ua = navigator.userAgent.toLowerCase();
        var isIE = (/msie/.test(ua)) && !(/opera/.test(ua)) && (/win/.test(ua));

        // create the style node for all browsers
        var style_node = document.getElementById("tx_vtnt_inc_mapLayer");
        try {
        	style_node.removeChild(style_node.childNodes[0]);
		} catch (e) {
			this.oLogger.info(e);
		}
		var styleText = "";	
		
		// Fading-Stufe 1
		if (this.getCurrentScale() <= 1.01)   {
           // styleText += ".mapLayer3 { opacity:0; } \n";
           // styleText += ".mapLayer4 { opacity:0; } \n";
            styleText += ".mapLayer1 { opacity:1.0; } \n";
            styleText += ".mapLayer2 { opacity:1.0; } \n";
        };

		if ( this.getCurrentScale() >= 1.01) // && this.getCurrentScale() < 1.2)
		{
			styleText += ".mapLayer1 { opacity:1.0; } \n";
			styleText += ".mapLayer2 { opacity:1.0; } \n";
		};
		
		if ( this.getCurrentScale() >= 1.01 ) {
          if (this.getCurrentScale() < 1.7) {
             styleText += ".mapLayer1 { opacity:1.0; } \n";
             //styleText += ".mapLayer2 { opacity:"+(1.7-(Math.abs(this.getCurrentScale()-1.01)*1.5))+";} \n"
        	} //else { styleText += ".mapLayer2 { opacity: 0.0;} \n"; }
        } else {
             //styleText += ".mapLayer1 { opacity:1.0; } \n";
			 //styleText += ".mapLayer2 { opacity: 1.0;} \n";
        }

  		// Fading-Stufe 2
        if (this.getCurrentScale() >= 1.2) {          
            if (this.getCurrentScale() < 1.8) {
            	styleText += " .mapLayer3 { opacity:"+(Math.abs(this.getCurrentScale()-1.2)*1.5)+";} \n";
            	// outzooming of lower layer
            	styleText += " .mapLayer2 .prod_txt { opacity:"+(1-(Math.abs(this.getCurrentScale()-1.1)*1.5))+";} \n";
            }
            else {
            	styleText += " .mapLayer3 { opacity:1.0;} \n"; 
	           	styleText += " .mapLayer2 .prod_txt{ opacity:0.0;} \n"; 
			}       
        } else {
        styleText += ".mapLayer3 { opacity:0; } \n";
        
        };
        
        if (this.getCurrentScale() > 1.8) {
        	    styleText += " .mapLayer1 { opacity:1.0;} \n";
        	    styleText += " .mapLayer2 .prod_txt { opacity:0.0;} \n";
        }

        // Fading-Stufe 3
        if (this.getCurrentScale() >= 1.5){

        	if (this.getCurrentScale() < 2.1) 
          	{ 
          		styleText += ".mapLayer4 { opacity:"+((this.getCurrentScale()-1.5)*1.5)+";} \n";
          	} else {
            	styleText += ".mapLayer4 { opacity:1.0;} \n";
          	};
          
        } else styleText += ".mapLayer4 { opacity:0.0; }  \n";
        
        // Fading-Stufe 4
        if (this.getCurrentScale() >= 2.0){

        	if (this.getCurrentScale() < 2.7) 
          	{ 
          		styleText += ".mapLayer5 { opacity:"+((this.getCurrentScale()-2)*1.5)+";} \n";
          		styleText += ".mapLayer3 .prod_txt{ opacity:"+(1-(this.getCurrentScale()-2)*1.5)+";} \n";
          	} else {
            	styleText += ".mapLayer5 { opacity:1.0;} \n";
            	styleText += ".mapLayer3 .prod_txt { opacity:0.0;} \n";
          	};
          
        } else styleText += ".mapLayer5 { opacity:0.0; }  \n";

 
 
 		//nur zum Test 3.Ebene immer anzeigen
        styleText += " .mapLayer3 { opacity:1.0;} \n";

        // append a rule for good browsers
       if (!isIE) style_node.appendChild(document.createTextNode(styleText));

	},
    
    
    getName: function() {
      return this.sName;
    }
  }
  Object.extend(VisualMap.prototype, Enumerable);
